# -*- coding: utf-8 -*-
from docopt import Dict as DocOptDict
from irc3.plugins.command import command
from irc3.utils import IrcString
import asyncio
import irc3

from . import Plugin
from ..utils import fmt, time_to_sec


@irc3.plugin
class Reminder(Plugin):
    requires = [
        'irc3.plugins.command',
    ]

    @command
    def remind(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Remind command.
        %%remind <delay> <message>...
        """
        delay = time_to_sec(args['<delay>'])
        if delay:
            asyncio.ensure_future(self.timer(delay, mask, channel, args))
        else:
            self.bot.notice(mask.nick, 'Invalid delay "{delay}" for "remind"'
                            .format(delay=args['<delay>']))

    async def timer(self, delay, mask: IrcString, channel: IrcString,
                    args: DocOptDict):
        """
        Actually the reminder function. Sleeps `delay` seconds and then sends
        message to `channel` after that.
        """
        await asyncio.sleep(delay)
        text = fmt('{bold}[Reminder]{reset} {nick}: {message} ({delay})',
                   message=' '.join(args['<message>']),
                   delay=args['<delay>'],
                   nick=mask.nick)
        self.bot.privmsg(channel, text)
