# -*- coding: utf-8 -*-
from requests import request, Response
from datetime import timedelta
from pprint import pprint
import re

# @formatter:off
COLORS = dict(
    white='00',    # white
    black='01',    # black
    blue='02',     # blue (navy)
    green='03',    # green
    red='04',      # red
    maroon='05',   # brown (maroon)
    purple='06',   # purple
    orange='07',   # orange (olive)
    yellow='08',   # yellow
    ltgreen='09',  # light green (lime)
    teal='10',     # teal (a green/blue cyan)
    ltcyan='11',   # light cyan (cyan / aqua)
    ltblue='12',   # light blue (royal)
    pink='13',     # pink (light purple / fuchsia)
    grey='14',     # grey
    ltgrey='15',   # light grey (silver)
)

FORMATTING = dict(
    bold='\x02',       # bold
    color='\x03',      # colored text
    italic='\x1D',     # italic text
    underline='\x1F',  # underlined text
    swap='\x16',       # swap bg and fg colors ("reverse video")
    reset='\x0F',      # reset all formatting
    **COLORS
)
# @formatter:on


TIME_REGEX = re.compile(r'(\d+)([smhdwy])')
TIME_DICT = {
    's': ['seconds', 1],
    'm': ['minutes', 60],
    'h': ['hours', 3600],
    'd': ['days', 86400],
    'w': ['weeks', 604800],
    'y': ['years', 31449600],
}

# Debug helper
pp = pprint


def fmt(__text: str, **kwargs) -> str:
    """Formats a str with `kwargs` and `FORMATTING`."""
    return __text.format(**kwargs, **FORMATTING)


async def req(method: str, url: str, **kwargs) -> Response:
    return request(method, url, **kwargs)


def time_to_sec(text: str) -> int:
    match = TIME_REGEX.match(text)
    if match:
        unit, num = match
        return num * TIME_DICT[unit][1]


def time_delta(text: str) -> timedelta:
    match = TIME_REGEX.match(text)
    if match:
        unit, num = match
        if unit == 'years':
            num *= 52
        return timedelta(**{TIME_DICT[unit][0]: num})


def parse_int(val: str, select: bool = True) -> tuple:
    try:
        val = int(val)
        if val is not 0:
            if val < 1:
                order = 'desc'
                val *= -1
                op = '<='
            else:
                order = 'asc'
                op = '>='
            if select:
                val -= 1
            return val, order, op
    except ValueError:
        pass


def _parse_int(val: list) -> int:
    """
    Parses an int from list, decremts by -1 if positiv.
    Only returns if value from list is not 0.
    """
    try:
        val = int(''.join(val))
        # Ignore val == 0
        if val is not 0:
            # Decrement to match list index
            if val > 0:
                val -= 1
            return val
    except ValueError:
        pass
